@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2020 Jon Evans <jon@craftyjon.com>
  * Copyright (C) 2023 CERN
- * Copyright (C) 2020-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -81,6 +81,35 @@ PARAM_LAYER_PRESET_3D::PARAM_LAYER_PRESET_3D( const std::string& aPath,
         m_presets( aPresetList )
 {
     wxASSERT( aPresetList );
+
+#define LAYER( n, l ) m_layerToLayerNameMap[l] = n; m_layerNameToLayerMap[n] = l;
+
+    LAYER( "fp_values",           LAYER_FP_VALUES            );
+    LAYER( "fp_references",       LAYER_FP_REFERENCES        );
+    LAYER( "fp_text",             LAYER_FP_TEXT              );
+    LAYER( "background_bottom",   LAYER_3D_BACKGROUND_BOTTOM );
+    LAYER( "background_top",      LAYER_3D_BACKGROUND_TOP    );
+    LAYER( "board",               LAYER_3D_BOARD             );
+    LAYER( "copper",              LAYER_3D_COPPER_TOP        );
+    LAYER( "copper_bottom",       LAYER_3D_COPPER_BOTTOM     );
+    LAYER( "silkscreen_bottom",   LAYER_3D_SILKSCREEN_BOTTOM );
+    LAYER( "silkscreen_top",      LAYER_3D_SILKSCREEN_TOP    );
+    LAYER( "soldermask_bottom",   LAYER_3D_SOLDERMASK_BOTTOM );
+    LAYER( "soldermask_top",      LAYER_3D_SOLDERMASK_TOP    );
+    LAYER( "solderpaste",         LAYER_3D_SOLDERPASTE       );
+    LAYER( "adhesive",            LAYER_3D_ADHESIVE          );
+    LAYER( "user_comments",       LAYER_3D_USER_COMMENTS     );
+    LAYER( "user_drawings",       LAYER_3D_USER_DRAWINGS     );
+    LAYER( "user_eco1",           LAYER_3D_USER_ECO1         );
+    LAYER( "user_eco2",           LAYER_3D_USER_ECO2         );
+    LAYER( "3d_axes",             LAYER_3D_AXES              );
+    LAYER( "th_models",           LAYER_3D_TH_MODELS         );
+    LAYER( "smd_models",          LAYER_3D_SMD_MODELS        );
+    LAYER( "virtual_models",      LAYER_3D_VIRTUAL_MODELS    );
+    LAYER( "non_pos_file_models", LAYER_3D_MODELS_NOT_IN_POS );
+    LAYER( "dnp_models",          LAYER_3D_MODELS_MARKED_DNP );
+    LAYER( "bounding_boxes",      LAYER_3D_BOUNDING_BOXES    );
+    LAYER( "off_board_silk",      LAYER_3D_OFF_BOARD_SILK    );
 }
 
 
@@ -99,7 +128,7 @@ nlohmann::json PARAM_LAYER_PRESET_3D::presetsToJson()
         for( int layer = 0; layer < LAYER_3D_END; ++layer )
         {
             if( preset.layers.test( layer ) )
-                layers.push_back( layer );
+                layers.push_back( m_layerToLayerNameMap[layer] );
         }
 
         js["layers"] = layers;
@@ -109,7 +138,7 @@ nlohmann::json PARAM_LAYER_PRESET_3D::presetsToJson()
         for( const auto& [ layer, color ] : preset.colors )
         {
             nlohmann::json layerColor = {
-                { "layer", layer },
+                { "layer", m_layerToLayerNameMap[layer] },
                 { "color", color.ToCSSString() }
             };
 
@@ -144,26 +173,20 @@ void PARAM_LAYER_PRESET_3D::jsonToPresets( const nlohmann::json& aJson )
 
                 for( const nlohmann::json& layer : preset.at( "layers" ) )
                 {
-                    if( layer.is_number_integer() )
-                    {
-                        int layerNum = layer.get<int>();
-
-                        if( layerNum >= 0 && layerNum < LAYER_3D_END )
-                            p.layers.set( layerNum );
-                    }
+                    if( layer.is_string() )
+                        p.layers.set( m_layerNameToLayerMap[layer.get<wxString>()] );
                 }
             }
 
             if( preset.contains( "colors" ) && preset.at( "colors" ).is_array() )
             {
-                for( const nlohmann::json& layerColor : preset.at( "colors" ) )
+                for( const nlohmann::json& entry : preset.at( "colors" ) )
                 {
-                    if( layerColor.contains( "layer" ) && layerColor.contains( "color" )
-                        && layerColor.at( "layer" ).is_number_integer() )
+                    if( entry.contains( "layer" ) && entry.contains( "color" )
+                        && entry.at( "layer" ).is_string() )
                     {
-                        int layerNum = layerColor.at( "layer" ).get<int>();
-                        COLOR4D color = layerColor.at( "color" ).get<COLOR4D>();
-                        p.colors[ layerNum ] = color;
+                        int layerNum = m_layerNameToLayerMap[entry.at( "layer" ).get<wxString>()];
+                        p.colors[ layerNum ] = entry.at( "color" ).get<COLOR4D>();
                     }
                 }
             }
@@ -175,7 +198,7 @@ void PARAM_LAYER_PRESET_3D::jsonToPresets( const nlohmann::json& aJson )
 
 
 ///! Update the schema version whenever a migration is required
-const int viewer3dSchemaVersion = 3;
+const int viewer3dSchemaVersion = 4;
 
 
 EDA_3D_VIEWER_SETTINGS::EDA_3D_VIEWER_SETTINGS() :
@@ -368,6 +391,8 @@ EDA_3D_VIEWER_SETTINGS::EDA_3D_VIEWER_SETTINGS() :
                                             &m_Render.clip_silk_on_via_annuli, false ) );
     m_params.emplace_back( new PARAM<bool>( "render.plated_and_bare_copper",
                                             &m_Render.differentiate_plated_copper, false ) );
+    m_params.emplace_back( new PARAM<bool>( "render.use_board_editor_copper_colors",
+                                            &m_Render.use_board_editor_copper_colors, false ) );
     m_params.emplace_back( new PARAM<bool>( "camera.animation_enabled",
                                             &m_Camera.animation_enabled, true ) );
     m_params.emplace_back( new PARAM<int>( "camera.moving_speed_multiplier",
@@ -377,6 +402,8 @@ EDA_3D_VIEWER_SETTINGS::EDA_3D_VIEWER_SETTINGS() :
     m_params.emplace_back( new PARAM<int>( "camera.projection_mode",
                                            &m_Camera.projection_mode, 1 ) );
 
+    m_params.emplace_back( new PARAM<bool>( "use_stackup_colors",
+                                            &m_UseStackupColors, true ) );
     m_params.emplace_back( new PARAM_LAYER_PRESET_3D( "layer_presets",
                                                       &m_LayerPresets ) );
     m_params.emplace_back( new PARAM<wxString>( "current_layer_preset",
@@ -421,6 +448,69 @@ EDA_3D_VIEWER_SETTINGS::EDA_3D_VIEWER_SETTINGS() :
                     Set( "render.show_eco2", *optval );
                 }
 
+                return true;
+            } );
+
+    registerMigration( 3, 4,
+            [&]() -> bool
+            {
+                std::map<int, wxString> legacyColorMap;
+
+                legacyColorMap[142] = "fp_values";
+                legacyColorMap[143] = "fp_references";
+                legacyColorMap[130] = "fp_text";
+                legacyColorMap[466] = "background_bottom";
+                legacyColorMap[467] = "background_top";
+                legacyColorMap[468] = "board";
+                legacyColorMap[469] = "copper";
+                legacyColorMap[470] = "copper_bottom";
+                legacyColorMap[471] = "silkscreen_bottom";
+                legacyColorMap[472] = "silkscreen_top";
+                legacyColorMap[473] = "soldermask_bottom";
+                legacyColorMap[474] = "soldermask_top";
+                legacyColorMap[475] = "solderpaste";
+                legacyColorMap[476] = "adhesive";
+                legacyColorMap[477] = "user_comments";
+                legacyColorMap[478] = "user_drawings";
+                legacyColorMap[479] = "user_eco1";
+                legacyColorMap[480] = "user_eco2";
+                legacyColorMap[481] = "th_models";
+                legacyColorMap[482] = "smd_models";
+                legacyColorMap[483] = "virtual_models";
+                legacyColorMap[484] = "non_pos_file_models";
+                legacyColorMap[485] = "dnp_models";
+                legacyColorMap[486] = "3d_axes";
+                legacyColorMap[487] = "bounding_boxes";
+                legacyColorMap[488] = "off_board_silk";
+
+                if( !Contains( "layer_presets" ) || !At( "layer_presets" ).is_array() )
+                    return true;
+
+                for( nlohmann::json& preset : At( "layer_presets" ) )
+                {
+                    if( preset.contains( "colors" ) && preset.at( "colors" ).is_array() )
+                    {
+                        for( nlohmann::json& color : preset.at( "colors" ) )
+                        {
+                            if( color.contains( "layer" ) && color.at( "layer" ).is_number_integer() )
+                                color["layer"] = legacyColorMap[color["layer"].get<int>()];
+                        }
+                    }
+
+                    if( preset.contains( "layers" ) && preset.at( "layers" ).is_array() )
+                    {
+                        nlohmann::json mappedLayers = nlohmann::json::array();
+
+                        for( const nlohmann::json& layer : preset.at( "layers" ) )
+                        {
+                            if( layer.is_number_integer() )
+                                mappedLayers.push_back( legacyColorMap[layer.get<int>()] );
+                        }
+
+                        preset["layers"] = mappedLayers;
+                    }
+                }
+
                 return true;
             } );
 }
@@ -465,12 +555,15 @@ bool EDA_3D_VIEWER_SETTINGS::MigrateFromLegacy( wxConfigBase* aCfg )
     ret &= fromLegacy<int>( aCfg,  "RenderEngine",              "render.engine" );
     ret &= fromLegacy<int>( aCfg,  "ShowGrid3D",                "render.grid_type" );
     ret &= fromLegacy<int>( aCfg,  "Render_Material",           "render.material_mode" );
-    ret &= fromLegacy<bool>( aCfg, "Render_OGL_ShowCopperThickness", "render.opengl_copper_thickness" );
-    ret &= fromLegacy<bool>( aCfg, "Render_OGL_ShowModelBoudingBoxes", "render.opengl_show_model_bbox" );
+    ret &= fromLegacy<bool>( aCfg, "Render_OGL_ShowCopperThickness",
+                             "render.opengl_copper_thickness" );
+    ret &= fromLegacy<bool>( aCfg, "Render_OGL_ShowModelBoudingBoxes",
+                             "render.opengl_show_model_bbox" );
     ret &= fromLegacy<bool>( aCfg, "Render_RAY_AntiAliasing",   "render.raytrace_anti_aliasing" );
     ret &= fromLegacy<bool>( aCfg, "Render_RAY_Backfloor",      "render.raytrace_backfloor" );
     ret &= fromLegacy<bool>( aCfg, "Render_RAY_PostProcess",    "render.raytrace_post_processing" );
-    ret &= fromLegacy<bool>( aCfg, "Render_RAY_ProceduralTextures", "render.raytrace_procedural_textures" );
+    ret &= fromLegacy<bool>( aCfg, "Render_RAY_ProceduralTextures",
+                             "render.raytrace_procedural_textures" );
     ret &= fromLegacy<bool>( aCfg, "Render_RAY_Reflections",    "render.raytrace_reflections" );
     ret &= fromLegacy<bool>( aCfg, "Render_RAY_Refractions",    "render.raytrace_refractions" );
     ret &= fromLegacy<bool>( aCfg, "Render_RAY_Shadows",        "render.raytrace_shadows" );
