@@ -22,30 +22,20 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-/*
- * Fields are texts attached to a symbol, some of which have a special meaning.
- * Fields 0 and 1 are very important: reference and value.
- * Field 2 is used as default footprint name.
- * Field 3 is used to point to a datasheet (usually a URL).
- * Fields 4+ are user fields.  They can be renamed and can appear in reports.
- */
-
 #include <wx/log.h>
 #include <wx/menu.h>
 
 #include <advanced_config.h>
 #include <base_units.h>
 #include <common.h>     // for ExpandTextVars
+#include <ctl_flags.h>
 #include <sch_edit_frame.h>
 #include <plotters/plotter.h>
+#include <richio.h>
+#include <io/kicad/kicad_io_utils.h>
 #include <bitmaps.h>
-#include <core/mirror.h>
 #include <kiway.h>
 #include <symbol_library.h>
-#include <sch_symbol.h>
-#include <sch_field.h>
-#include <sch_label.h>
-#include <schematic.h>
 #include <settings/color_settings.h>
 #include <string_utils.h>
 #include <trace_helpers.h>
@@ -83,18 +73,26 @@ SCH_FIELD::SCH_FIELD( const VECTOR2I& aPos, int aFieldId, SCH_ITEM* aParent,
 }
 
 
-SCH_FIELD::SCH_FIELD( SCH_ITEM* aParent, int aFieldId, const wxString& aName) :
+SCH_FIELD::SCH_FIELD( SCH_ITEM* aParent, int aFieldId, const wxString& aName ) :
         SCH_FIELD( VECTOR2I(), aFieldId, aParent, aName )
 {
 }
 
 
+SCH_FIELD::SCH_FIELD( SCH_TEXT* aText ) :
+        SCH_FIELD( VECTOR2I(), INVALID_FIELD, nullptr, wxEmptyString )
+{
+    SCH_ITEM::operator=( *aText );
+    EDA_TEXT::operator=( *aText );
+}
+
+
 SCH_FIELD::SCH_FIELD( const SCH_FIELD& aField ) :
         SCH_ITEM( aField ),
         EDA_TEXT( aField )
 {
     m_private         = aField.m_private;
-    m_id              = aField.m_id;
+    SetId( aField.m_id );  // will also set the layer
     m_name            = aField.m_name;
     m_showName        = aField.m_showName;
     m_allowAutoPlace  = aField.m_allowAutoPlace;
@@ -124,7 +122,7 @@ SCH_FIELD& SCH_FIELD::operator=( const SCH_FIELD& aField )
     EDA_TEXT::operator=( aField );
 
     m_private         = aField.m_private;
-    m_id              = aField.m_id;
+    SetId( aField.m_id );  // will also set the layer
     m_name            = aField.m_name;
     m_showName        = aField.m_showName;
     m_allowAutoPlace  = aField.m_allowAutoPlace;
@@ -512,9 +510,92 @@ void SCH_FIELD::Print( const SCH_RENDER_SETTINGS* aSettings, int aUnit, int aBod
 }
 
 
+// The following is taken from EDA_TEXT::Format because after 9.0.0, the `(hide yes)` property
+// was removed from EDA_TEXT but not EDA_FIELD, but to preserve compatibility with 9.0.0, it
+// needs to be written inside the effects block.
+void SCH_FIELD::Format( OUTPUTFORMATTER* aFormatter, int aControlBits ) const
+{
+    aFormatter->Print( "(effects" );
+
+    aFormatter->Print( "(font" );
+
+    if( GetFont() && !GetFont()->GetName().IsEmpty() )
+        aFormatter->Print( "(face %s)", aFormatter->Quotew( GetFont()->NameAsToken() ).c_str() );
+
+    // Text size
+    aFormatter->Print( "(size %s %s)",
+                       EDA_UNIT_UTILS::FormatInternalUnits( m_IuScale, GetTextHeight() ).c_str(),
+                       EDA_UNIT_UTILS::FormatInternalUnits( m_IuScale, GetTextWidth() ).c_str() );
+
+    if( GetLineSpacing() != 1.0 )
+    {
+        aFormatter->Print( "(line_spacing %s)",
+                           FormatDouble2Str( GetLineSpacing() ).c_str() );
+    }
+
+    if( GetTextThickness() )
+    {
+        aFormatter->Print( "(thickness %s)",
+                EDA_UNIT_UTILS::FormatInternalUnits( m_IuScale, GetTextThickness() ).c_str() );
+    }
+
+    if( IsBold() )
+        KICAD_FORMAT::FormatBool( aFormatter, "bold", true );
+
+    if( IsItalic() )
+        KICAD_FORMAT::FormatBool( aFormatter, "italic", true );
+
+    if( !( aControlBits & CTL_OMIT_COLOR ) && GetTextColor() != COLOR4D::UNSPECIFIED )
+    {
+        aFormatter->Print( "(color %d %d %d %s)",
+                           KiROUND( GetTextColor().r * 255.0 ),
+                           KiROUND( GetTextColor().g * 255.0 ),
+                           KiROUND( GetTextColor().b * 255.0 ),
+                           FormatDouble2Str( GetTextColor().a ).c_str() );
+    }
+
+    aFormatter->Print( ")"); // (font
+
+    if( IsMirrored() || GetHorizJustify() != GR_TEXT_H_ALIGN_CENTER
+                     || GetVertJustify() != GR_TEXT_V_ALIGN_CENTER )
+    {
+        aFormatter->Print( "(justify");
+
+        if( GetHorizJustify() != GR_TEXT_H_ALIGN_CENTER )
+            aFormatter->Print( GetHorizJustify() == GR_TEXT_H_ALIGN_LEFT ? " left" : " right" );
+
+        if( GetVertJustify() != GR_TEXT_V_ALIGN_CENTER )
+            aFormatter->Print( GetVertJustify() == GR_TEXT_V_ALIGN_TOP ? " top" : " bottom" );
+
+        if( IsMirrored() )
+            aFormatter->Print( " mirror" );
+
+        aFormatter->Print( ")" ); // (justify
+    }
+
+    // The relevant difference from EDA_TEXT::Format
+    if( !IsVisible() )
+        KICAD_FORMAT::FormatBool( aFormatter, "hide", true );
+
+    if( !( aControlBits & CTL_OMIT_HYPERLINK ) && HasHyperlink() )
+        aFormatter->Print( "(href %s)", aFormatter->Quotew( GetHyperlink() ).c_str() );
+
+    aFormatter->Print( ")" ); // (effects
+}
+
+
+bool SCH_FIELD::IsDefaultFormatting() const
+{
+    return ( IsVisible()
+             && EDA_TEXT::IsDefaultFormatting()
+           );
+}
+
+
 void SCH_FIELD::ImportValues( const SCH_FIELD& aSource )
 {
     SetAttributes( aSource );
+    SetVisible( aSource.IsVisible() );
     SetNameShown( aSource.IsNameShown() );
     SetCanAutoplace( aSource.CanAutoplace() );
 }
@@ -862,7 +943,7 @@ void SCH_FIELD::OnScintillaCharAdded( SCINTILLA_TRICKS* aScintillaTricks,
                 {
                     NULL_REPORTER   devnull;
                     SCH_SHEET_PATH& sheet = schematic->CurrentSheet();
-                    SIM_LIB_MGR     mgr( &schematic->Prj() );
+                    SIM_LIB_MGR     mgr( &schematic->Prj(), schematic );
                     SIM_MODEL&      model = mgr.CreateModel( &sheet, *symbol, devnull ).model;
 
                     for( wxString pin : model.GetPinNames() )
