@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2023 Alex Shvartzkop <dudesuchamazing@gmail.com>
- * Copyright (C) 2023-2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,21 +27,16 @@
 #include <sch_io/sch_io_mgr.h>
 #include <schematic.h>
 #include <sch_sheet.h>
-#include <sch_sheet_pin.h>
 #include <sch_line.h>
-#include <lib_shape.h>
-#include <lib_text.h>
 #include <sch_no_connect.h>
 #include <sch_label.h>
 #include <sch_junction.h>
 #include <sch_edit_frame.h>
 #include <sch_shape.h>
 #include <sch_bitmap.h>
-#include <sch_bus_entry.h>
 #include <string_utils.h>
 #include <bezier_curves.h>
 #include <wx/base64.h>
-#include <wx/url.h>
 #include <wx/mstream.h>
 #include <core/map_helpers.h>
 #include <gfx_import_utils.h>
@@ -195,48 +190,48 @@ VECTOR2I HelperGeneratePowerPortGraphics( LIB_SYMBOL* aKsymbol, EASYEDA::POWER_F
 {
     if( aStyle == EASYEDA::POWER_FLAG_STYLE::CIRCLE || aStyle == EASYEDA::POWER_FLAG_STYLE::ARROW )
     {
-        LIB_SHAPE* line1 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-        aKsymbol->AddDrawItem( line1 );
+        SCH_SHAPE* line1 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
         line1->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
         line1->AddPoint( { 0, 0 } );
-        line1->AddPoint( { 0, schIUScale.MilsToIU( -50 ) } );
+        line1->AddPoint( { 0, schIUScale.MilsToIU( 50 ) } );
+        aKsymbol->AddDrawItem( line1, false );
 
         if( aStyle == EASYEDA::POWER_FLAG_STYLE::CIRCLE )
         {
-            LIB_SHAPE* circle = new LIB_SHAPE( aKsymbol, SHAPE_T::CIRCLE );
-            aKsymbol->AddDrawItem( circle );
+            SCH_SHAPE* circle = new SCH_SHAPE( SHAPE_T::CIRCLE, LAYER_DEVICE );
             circle->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 5 ), LINE_STYLE::SOLID ) );
-            circle->SetPosition( { schIUScale.MilsToIU( 0 ), schIUScale.MilsToIU( -75 ) } );
+            circle->SetPosition( { schIUScale.MilsToIU( 0 ), schIUScale.MilsToIU( 75 ) } );
             circle->SetEnd( circle->GetPosition() + VECTOR2I( schIUScale.MilsToIU( 25 ), 0 ) );
+            aKsymbol->AddDrawItem( circle, false );
         }
         else
         {
-            LIB_SHAPE* line2 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-            aKsymbol->AddDrawItem( line2 );
+            SCH_SHAPE* line2 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
             line2->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-            line2->AddPoint( { schIUScale.MilsToIU( -25 ), schIUScale.MilsToIU( -50 ) } );
-            line2->AddPoint( { schIUScale.MilsToIU( 25 ), schIUScale.MilsToIU( -50 ) } );
-            line2->AddPoint( { schIUScale.MilsToIU( 0 ), schIUScale.MilsToIU( -100 ) } );
-            line2->AddPoint( { schIUScale.MilsToIU( -25 ), schIUScale.MilsToIU( -50 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( -25 ), schIUScale.MilsToIU( 50 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( 25 ), schIUScale.MilsToIU( 50 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( 0 ), schIUScale.MilsToIU( 100 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( -25 ), schIUScale.MilsToIU( 50 ) } );
+            aKsymbol->AddDrawItem( line2, false );
         }
 
         return { 0, schIUScale.MilsToIU( 150 ) };
     }
     else if( aStyle == EASYEDA::POWER_FLAG_STYLE::WAVE )
     {
-        LIB_SHAPE* line = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-        aKsymbol->AddDrawItem( line );
+        SCH_SHAPE* line = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
         line->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
         line->AddPoint( { 0, 0 } );
-        line->AddPoint( { 0, schIUScale.MilsToIU( -72 ) } );
+        line->AddPoint( { 0, schIUScale.MilsToIU( 72 ) } );
+        aKsymbol->AddDrawItem( line, false );
 
-        LIB_SHAPE* bezier = new LIB_SHAPE( aKsymbol, SHAPE_T::BEZIER );
-        aKsymbol->AddDrawItem( bezier );
+        SCH_SHAPE* bezier = new SCH_SHAPE( SHAPE_T::BEZIER, LAYER_DEVICE );
         bezier->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 5 ), LINE_STYLE::SOLID ) );
-        bezier->AddPoint( { schIUScale.MilsToIU( 30 ), schIUScale.MilsToIU( -50 ) } );
-        bezier->AddPoint( { schIUScale.MilsToIU( 30 ), schIUScale.MilsToIU( -87 ) } );
-        bezier->AddPoint( { schIUScale.MilsToIU( -30 ), schIUScale.MilsToIU( -63 ) } );
-        bezier->AddPoint( { schIUScale.MilsToIU( -30 ), schIUScale.MilsToIU( -100 ) } );
+        bezier->SetStart( { schIUScale.MilsToIU( 30 ), schIUScale.MilsToIU( 50 ) } );
+        bezier->SetBezierC1( { schIUScale.MilsToIU( 30 ), schIUScale.MilsToIU( 87 ) } );
+        bezier->SetBezierC2( { schIUScale.MilsToIU( -30 ), schIUScale.MilsToIU( 63 ) } );
+        bezier->SetEnd( { schIUScale.MilsToIU( -30 ), schIUScale.MilsToIU( 100 ) } );
+        aKsymbol->AddDrawItem( bezier, false );
 
         return { 0, schIUScale.MilsToIU( 150 ) };
     }
@@ -245,72 +240,72 @@ VECTOR2I HelperGeneratePowerPortGraphics( LIB_SYMBOL* aKsymbol, EASYEDA::POWER_F
              || aStyle == EASYEDA::POWER_FLAG_STYLE::EARTH
              || aStyle == EASYEDA::POWER_FLAG_STYLE::GOST_ARROW )
     {
-        LIB_SHAPE* line1 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-        aKsymbol->AddDrawItem( line1 );
+        SCH_SHAPE* line1 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
         line1->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
         line1->AddPoint( { 0, 0 } );
-        line1->AddPoint( { 0, schIUScale.MilsToIU( -100 ) } );
+        line1->AddPoint( { 0, schIUScale.MilsToIU( 100 ) } );
+        aKsymbol->AddDrawItem( line1, false );
 
         if( aStyle == EASYEDA::POWER_FLAG_STYLE::POWER_GROUND )
         {
-            LIB_SHAPE* line2 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-            aKsymbol->AddDrawItem( line2 );
+            SCH_SHAPE* line2 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
             line2->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-            line2->AddPoint( { schIUScale.MilsToIU( -100 ), schIUScale.MilsToIU( -100 ) } );
-            line2->AddPoint( { schIUScale.MilsToIU( 100 ), schIUScale.MilsToIU( -100 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( -100 ), schIUScale.MilsToIU( 100 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( 100 ), schIUScale.MilsToIU( 100 ) } );
+            aKsymbol->AddDrawItem( line2, false );
 
-            LIB_SHAPE* line3 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-            aKsymbol->AddDrawItem( line3 );
+            SCH_SHAPE* line3 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
             line3->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-            line3->AddPoint( { schIUScale.MilsToIU( -70 ), schIUScale.MilsToIU( -120 ) } );
-            line3->AddPoint( { schIUScale.MilsToIU( 70 ), schIUScale.MilsToIU( -120 ) } );
+            line3->AddPoint( { schIUScale.MilsToIU( -70 ), schIUScale.MilsToIU( 120 ) } );
+            line3->AddPoint( { schIUScale.MilsToIU( 70 ), schIUScale.MilsToIU( 120 ) } );
+            aKsymbol->AddDrawItem( line3, false );
 
-            LIB_SHAPE* line4 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-            aKsymbol->AddDrawItem( line4 );
+            SCH_SHAPE* line4 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
             line4->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-            line4->AddPoint( { schIUScale.MilsToIU( -40 ), schIUScale.MilsToIU( -140 ) } );
-            line4->AddPoint( { schIUScale.MilsToIU( 40 ), schIUScale.MilsToIU( -140 ) } );
+            line4->AddPoint( { schIUScale.MilsToIU( -40 ), schIUScale.MilsToIU( 140 ) } );
+            line4->AddPoint( { schIUScale.MilsToIU( 40 ), schIUScale.MilsToIU( 140 ) } );
+            aKsymbol->AddDrawItem( line4, false );
 
-            LIB_SHAPE* line5 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-            aKsymbol->AddDrawItem( line5 );
+            SCH_SHAPE* line5 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
             line5->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-            line5->AddPoint( { schIUScale.MilsToIU( -10 ), schIUScale.MilsToIU( -160 ) } );
-            line5->AddPoint( { schIUScale.MilsToIU( 10 ), schIUScale.MilsToIU( -160 ) } );
+            line5->AddPoint( { schIUScale.MilsToIU( -10 ), schIUScale.MilsToIU( 160 ) } );
+            line5->AddPoint( { schIUScale.MilsToIU( 10 ), schIUScale.MilsToIU( 160 ) } );
+            aKsymbol->AddDrawItem( line5, false );
         }
         else if( aStyle == EASYEDA::POWER_FLAG_STYLE::SIGNAL_GROUND )
         {
-            LIB_SHAPE* line2 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-            aKsymbol->AddDrawItem( line2 );
+            SCH_SHAPE* line2 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
             line2->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-            line2->AddPoint( { schIUScale.MilsToIU( -100 ), schIUScale.MilsToIU( -100 ) } );
-            line2->AddPoint( { schIUScale.MilsToIU( 100 ), schIUScale.MilsToIU( -100 ) } );
-            line2->AddPoint( { schIUScale.MilsToIU( 0 ), schIUScale.MilsToIU( -160 ) } );
-            line2->AddPoint( { schIUScale.MilsToIU( -100 ), schIUScale.MilsToIU( -100 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( -100 ), schIUScale.MilsToIU( 100 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( 100 ), schIUScale.MilsToIU( 100 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( 0 ), schIUScale.MilsToIU( 160 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( -100 ), schIUScale.MilsToIU( 100 ) } );
+            aKsymbol->AddDrawItem( line2, false );
         }
         else if( aStyle == EASYEDA::POWER_FLAG_STYLE::EARTH )
         {
-            LIB_SHAPE* line2 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-            aKsymbol->AddDrawItem( line2 );
+            SCH_SHAPE* line2 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
             line2->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-            line2->AddPoint( { schIUScale.MilsToIU( -150 ), schIUScale.MilsToIU( -200 ) } );
-            line2->AddPoint( { schIUScale.MilsToIU( -100 ), schIUScale.MilsToIU( -100 ) } );
-            line2->AddPoint( { schIUScale.MilsToIU( 100 ), schIUScale.MilsToIU( -100 ) } );
-            line2->AddPoint( { schIUScale.MilsToIU( 50 ), schIUScale.MilsToIU( -200 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( -150 ), schIUScale.MilsToIU( 200 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( -100 ), schIUScale.MilsToIU( 100 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( 100 ), schIUScale.MilsToIU( 100 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( 50 ), schIUScale.MilsToIU( 200 ) } );
+            aKsymbol->AddDrawItem( line2, false );
 
-            LIB_SHAPE* line3 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-            aKsymbol->AddDrawItem( line3 );
+            SCH_SHAPE* line3 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
             line3->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-            line3->AddPoint( { schIUScale.MilsToIU( 0 ), schIUScale.MilsToIU( -100 ) } );
-            line3->AddPoint( { schIUScale.MilsToIU( -50 ), schIUScale.MilsToIU( -200 ) } );
+            line3->AddPoint( { schIUScale.MilsToIU( 0 ), schIUScale.MilsToIU( 100 ) } );
+            line3->AddPoint( { schIUScale.MilsToIU( -50 ), schIUScale.MilsToIU( 200 ) } );
+            aKsymbol->AddDrawItem( line3, false );
         }
         else // EASYEDA::POWER_FLAG_STYLE::GOST_ARROW
         {
-            LIB_SHAPE* line2 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-            aKsymbol->AddDrawItem( line2 );
+            SCH_SHAPE* line2 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
             line2->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-            line2->AddPoint( { schIUScale.MilsToIU( -25 ), schIUScale.MilsToIU( -50 ) } );
-            line2->AddPoint( { schIUScale.MilsToIU( 0 ), schIUScale.MilsToIU( -100 ) } );
-            line2->AddPoint( { schIUScale.MilsToIU( 25 ), schIUScale.MilsToIU( -50 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( -25 ), schIUScale.MilsToIU( 50 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( 0 ), schIUScale.MilsToIU( 100 ) } );
+            line2->AddPoint( { schIUScale.MilsToIU( 25 ), schIUScale.MilsToIU( 50 ) } );
+            aKsymbol->AddDrawItem( line2, false );
 
             return { 0, schIUScale.MilsToIU( 150 ) }; // special case
         }
@@ -320,54 +315,54 @@ VECTOR2I HelperGeneratePowerPortGraphics( LIB_SYMBOL* aKsymbol, EASYEDA::POWER_F
     else if( aStyle == EASYEDA::POWER_FLAG_STYLE::GOST_POWER_GROUND
              || aStyle == EASYEDA::POWER_FLAG_STYLE::GOST_EARTH )
     {
-        LIB_SHAPE* line1 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-        aKsymbol->AddDrawItem( line1 );
+        SCH_SHAPE* line1 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
         line1->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
         line1->AddPoint( { 0, 0 } );
-        line1->AddPoint( { 0, schIUScale.MilsToIU( -160 ) } );
+        line1->AddPoint( { 0, schIUScale.MilsToIU( 160 ) } );
+        aKsymbol->AddDrawItem( line1, false );
 
-        LIB_SHAPE* line2 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-        aKsymbol->AddDrawItem( line2 );
+        SCH_SHAPE* line2 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
         line2->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-        line2->AddPoint( { schIUScale.MilsToIU( -100 ), schIUScale.MilsToIU( -160 ) } );
-        line2->AddPoint( { schIUScale.MilsToIU( 100 ), schIUScale.MilsToIU( -160 ) } );
+        line2->AddPoint( { schIUScale.MilsToIU( -100 ), schIUScale.MilsToIU( 160 ) } );
+        line2->AddPoint( { schIUScale.MilsToIU( 100 ), schIUScale.MilsToIU( 160 ) } );
+        aKsymbol->AddDrawItem( line2, false );
 
-        LIB_SHAPE* line3 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-        aKsymbol->AddDrawItem( line3 );
+        SCH_SHAPE* line3 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
         line3->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-        line3->AddPoint( { schIUScale.MilsToIU( -60 ), schIUScale.MilsToIU( -200 ) } );
-        line3->AddPoint( { schIUScale.MilsToIU( 60 ), schIUScale.MilsToIU( -200 ) } );
+        line3->AddPoint( { schIUScale.MilsToIU( -60 ), schIUScale.MilsToIU( 200 ) } );
+        line3->AddPoint( { schIUScale.MilsToIU( 60 ), schIUScale.MilsToIU( 200 ) } );
+        aKsymbol->AddDrawItem( line3, false );
 
-        LIB_SHAPE* line4 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-        aKsymbol->AddDrawItem( line4 );
+        SCH_SHAPE* line4 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
         line4->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-        line4->AddPoint( { schIUScale.MilsToIU( -20 ), schIUScale.MilsToIU( -240 ) } );
-        line4->AddPoint( { schIUScale.MilsToIU( 20 ), schIUScale.MilsToIU( -240 ) } );
+        line4->AddPoint( { schIUScale.MilsToIU( -20 ), schIUScale.MilsToIU( 240 ) } );
+        line4->AddPoint( { schIUScale.MilsToIU( 20 ), schIUScale.MilsToIU( 240 ) } );
+        aKsymbol->AddDrawItem( line4, false );
 
         if( aStyle == EASYEDA::POWER_FLAG_STYLE::GOST_POWER_GROUND )
-            return { 0, schIUScale.MilsToIU( 300 ) };
+            return { 0, schIUScale.MilsToIU( -300 ) };
 
-        LIB_SHAPE* circle = new LIB_SHAPE( aKsymbol, SHAPE_T::CIRCLE );
-        aKsymbol->AddDrawItem( circle );
+        SCH_SHAPE* circle = new SCH_SHAPE( SHAPE_T::CIRCLE, LAYER_DEVICE );
         circle->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-        circle->SetPosition( { schIUScale.MilsToIU( 0 ), schIUScale.MilsToIU( -160 ) } );
+        circle->SetPosition( { schIUScale.MilsToIU( 0 ), schIUScale.MilsToIU( 160 ) } );
         circle->SetEnd( circle->GetPosition() + VECTOR2I( schIUScale.MilsToIU( 120 ), 0 ) );
+        aKsymbol->AddDrawItem( circle, false );
 
         return { 0, schIUScale.MilsToIU( 350 ) };
     }
     else if( aStyle == EASYEDA::POWER_FLAG_STYLE::GOST_BAR )
     {
-        LIB_SHAPE* line1 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-        aKsymbol->AddDrawItem( line1 );
+        SCH_SHAPE* line1 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
         line1->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
         line1->AddPoint( { 0, 0 } );
-        line1->AddPoint( { 0, schIUScale.MilsToIU( -200 ) } );
+        line1->AddPoint( { 0, schIUScale.MilsToIU( 200 ) } );
+        aKsymbol->AddDrawItem( line1, false );
 
-        LIB_SHAPE* line2 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-        aKsymbol->AddDrawItem( line2 );
+        SCH_SHAPE* line2 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
         line2->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-        line2->AddPoint( { schIUScale.MilsToIU( -100 ), schIUScale.MilsToIU( -200 ) } );
-        line2->AddPoint( { schIUScale.MilsToIU( 100 ), schIUScale.MilsToIU( -200 ) } );
+        line2->AddPoint( { schIUScale.MilsToIU( -100 ), schIUScale.MilsToIU( 200 ) } );
+        line2->AddPoint( { schIUScale.MilsToIU( 100 ), schIUScale.MilsToIU( 200 ) } );
+        aKsymbol->AddDrawItem( line2, false );
 
         return { 0, schIUScale.MilsToIU( 250 ) };
     }
@@ -379,17 +374,17 @@ VECTOR2I HelperGeneratePowerPortGraphics( LIB_SYMBOL* aKsymbol, EASYEDA::POWER_F
                                RPT_SEVERITY_WARNING );
         }
 
-        LIB_SHAPE* line1 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-        aKsymbol->AddDrawItem( line1 );
+        SCH_SHAPE* line1 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
         line1->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
         line1->AddPoint( { 0, 0 } );
-        line1->AddPoint( { 0, schIUScale.MilsToIU( -100 ) } );
+        line1->AddPoint( { 0, schIUScale.MilsToIU( 100 ) } );
+        aKsymbol->AddDrawItem( line1, false );
 
-        LIB_SHAPE* line2 = new LIB_SHAPE( aKsymbol, SHAPE_T::POLY );
-        aKsymbol->AddDrawItem( line2 );
+        SCH_SHAPE* line2 = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
         line2->SetStroke( STROKE_PARAMS( schIUScale.MilsToIU( 10 ), LINE_STYLE::SOLID ) );
-        line2->AddPoint( { schIUScale.MilsToIU( -50 ), schIUScale.MilsToIU( -100 ) } );
-        line2->AddPoint( { schIUScale.MilsToIU( 50 ), schIUScale.MilsToIU( -100 ) } );
+        line2->AddPoint( { schIUScale.MilsToIU( -50 ), schIUScale.MilsToIU( 100 ) } );
+        line2->AddPoint( { schIUScale.MilsToIU( 50 ), schIUScale.MilsToIU( 100 ) } );
+        aKsymbol->AddDrawItem( line2, false );
 
         return { 0, schIUScale.MilsToIU( 150 ) };
     }
@@ -422,7 +417,7 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
                         RelPosSym( VECTOR2I( Convert( ptArr[i - 1] ), Convert( ptArr[i] ) ) ) );
             }
 
-            std::unique_ptr<LIB_SHAPE> line = std::make_unique<LIB_SHAPE>( aSymbol, SHAPE_T::POLY );
+            auto line = std::make_unique<SCH_SHAPE>( SHAPE_T::POLY, LAYER_DEVICE );
 
             if( elType == wxS( "PG" ) )
                 chain.SetClosed( true );
@@ -465,10 +460,7 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
 
             for( SHAPE_LINE_CHAIN outline : lineChains )
             {
-                std::unique_ptr<LIB_SHAPE> shape =
-                        std::make_unique<LIB_SHAPE>( aSymbol, SHAPE_T::POLY );
-
-                outline.Mirror( false, true );
+                auto shape = std::make_unique<SCH_SHAPE>( SHAPE_T::POLY, LAYER_DEVICE );
 
                 if( outline.IsClosed() )
                     outline.Append( outline.CPoint( 0 ), true );
@@ -525,7 +517,7 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
                                          svgImportPlugin.GetImageHeight() );
 
                         VECTOR2D pixelScale( schIUScale.IUTomm( ScaleSize( size.x ) ) / imSize.x,
-                                             schIUScale.IUTomm( -ScaleSize( size.y ) ) / imSize.y );
+                                             schIUScale.IUTomm( ScaleSize( size.y ) ) / imSize.y );
 
                         libsymImporter.SetScale( pixelScale );
 
@@ -537,7 +529,7 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
                         svgImportPlugin.Import();
 
                         for( std::unique_ptr<EDA_ITEM>& item : libsymImporter.GetItems() )
-                            aSymbol->AddDrawItem( static_cast<LIB_ITEM*>( item.release() ) );
+                            aSymbol->AddDrawItem( static_cast<SCH_ITEM*>( item.release() ) );
                     }
                     else
                     {
@@ -558,7 +550,7 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
                             }
 
                             VECTOR2D pixelScale( ScaleSize( size.x ) / img.GetWidth(),
-                                                 -ScaleSize( size.y ) / img.GetHeight() );
+                                                 ScaleSize( size.y ) / img.GetHeight() );
 
                             ConvertImageToLibShapes( aSymbol, 0, img, pixelScale,
                                                      RelPosSym( start ) );
@@ -581,7 +573,7 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
 
             auto transform = []( VECTOR2I aVec )
             {
-                return VECTOR2I( aVec.x, -aVec.y );
+                return VECTOR2I( aVec.x, aVec.y );
             };
 
             for( const SHAPE_LINE_CHAIN& chain : chains )
@@ -592,8 +584,8 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
                     {
                         SHAPE_ARC arc = chain.Arc( chain.ArcIndex( i ) );
 
-                        std::unique_ptr<LIB_SHAPE> shape =
-                                std::make_unique<LIB_SHAPE>( aSymbol, SHAPE_T::ARC );
+                        std::unique_ptr<SCH_SHAPE> shape =
+                                std::make_unique<SCH_SHAPE>( SHAPE_T::ARC, LAYER_DEVICE );
 
                         shape->SetArcGeometry( transform( arc.GetP0() ),
                                                transform( arc.GetArcMid() ),
@@ -618,8 +610,8 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
                     {
                         SEG seg = chain.CSegment( i );
 
-                        std::unique_ptr<LIB_SHAPE> shape =
-                                std::make_unique<LIB_SHAPE>( aSymbol, SHAPE_T::POLY );
+                        std::unique_ptr<SCH_SHAPE> shape =
+                                std::make_unique<SCH_SHAPE>( SHAPE_T::POLY, LAYER_DEVICE );
 
                         shape->AddPoint( transform( seg.A ) );
                         shape->AddPoint( transform( seg.B ) );
@@ -649,8 +641,7 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
 
             //if( cr.x == 0 )
             {
-                std::unique_ptr<LIB_SHAPE> rect =
-                        std::make_unique<LIB_SHAPE>( aSymbol, SHAPE_T::RECTANGLE );
+                auto rect = std::make_unique<SCH_SHAPE>( SHAPE_T::RECTANGLE, LAYER_DEVICE );
 
                 rect->SetStart( RelPosSym( start ) );
                 rect->SetEnd( RelPosSym( start + size ) );
@@ -674,8 +665,7 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
         }
         else if( elType == wxS( "E" ) )
         {
-            std::unique_ptr<LIB_SHAPE> circle =
-                    std::make_unique<LIB_SHAPE>( aSymbol, SHAPE_T::CIRCLE );
+            auto circle = std::make_unique<SCH_SHAPE>( SHAPE_T::CIRCLE, LAYER_DEVICE );
 
             VECTOR2D   center( Convert( arr[1] ), Convert( arr[2] ) );
             VECTOR2D   radius( Convert( arr[3] ), Convert( arr[4] ) ); // TODO: corner radius
@@ -735,8 +725,8 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
 
             // M360,290h10 or M 420 300 h -5
             wxString lineData = pinPathColorParts[0];
-            wxRegEx  regex(
-                    wxS( "^M\\s*([-\\d.]+)[,\\s]([-\\d.]+)\\s*([h|v])\\s*([-\\d.]+)\\s*$" ) );
+            wxRegEx  regex( wxS( "^M\\s*([-\\d.]+)[,\\s]([-\\d.]+)\\s*([h|v])\\s*([-\\d.]+)\\s*$" ) );
+
             if( regex.Matches( lineData ) )
             {
                 startPoint.x = Convert( regex.GetMatch( lineData, 1 ) );
@@ -796,8 +786,7 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
                     {
                         VECTOR2D dotPos( Convert( dotParts[1] ), Convert( dotParts[2] ) );
 
-                        std::unique_ptr<LIB_SHAPE> circle =
-                                std::make_unique<LIB_SHAPE>( aSymbol, SHAPE_T::CIRCLE );
+                        auto circle = std::make_unique<SCH_SHAPE>( SHAPE_T::CIRCLE, LAYER_DEVICE );
 
                         circle->SetCenter( RelPosSym( dotPos ) );
                         circle->SetEnd( RelPosSym(
@@ -815,10 +804,7 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
 
                         for( SHAPE_LINE_CHAIN outline : lineChains )
                         {
-                            std::unique_ptr<LIB_SHAPE> shape =
-                                    std::make_unique<LIB_SHAPE>( aSymbol, SHAPE_T::POLY );
-
-                            outline.Mirror( false, true );
+                            auto shape = std::make_unique<SCH_SHAPE>( SHAPE_T::POLY, LAYER_DEVICE );
 
                             if( outline.IsClosed() )
                                 outline.Append( outline.CPoint( 0 ), true );
@@ -834,7 +820,7 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
                 }
             }
 
-            std::unique_ptr<LIB_PIN> pin = std::make_unique<LIB_PIN>( aSymbol );
+            std::unique_ptr<SCH_PIN> pin = std::make_unique<SCH_PIN>( aSymbol );
 
             pin->SetName( pinName );
             pin->SetNumber( pinNumber );
@@ -878,20 +864,21 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
             if( textType == wxS( "P" ) )
             {
                 textItem = &aSymbol->GetReferenceField();
+                textItem->SetTextPos( RelPosSym( pos ) );
+                textItem->SetText( textStr );
             }
             else if( textType == wxS( "N" ) )
             {
                 textItem = &aSymbol->GetValueField();
+                textItem->SetTextPos( RelPosSym( pos ) );
+                textItem->SetText( textStr );
             }
             else
             {
-                textItem = new LIB_TEXT( aSymbol );
+                textItem = new SCH_TEXT( RelPosSym( pos ), textStr, LAYER_DEVICE );
                 added = true;
             }
 
-            textItem->SetTextPos( RelPosSym( pos ) );
-            textItem->SetText( textStr );
-
             textItem->SetTextAngleDegrees( ( 360 - angle ) % 360 );
             textItem->SetVertJustify( GR_TEXT_V_ALIGN_TOP );
 
@@ -924,10 +911,10 @@ void SCH_EASYEDA_PARSER::ParseSymbolShapes( LIB_SYMBOL*                  aSymbol
 
             textItem->SetTextSize( VECTOR2I( ktextSize, ktextSize ) );
 
-            TransformTextToBaseline( textItem, baselineAlign, true );
+            TransformTextToBaseline( textItem, baselineAlign );
 
             if( added )
-                aSymbol->AddDrawItem( dynamic_cast<LIB_ITEM*>( textItem ) );
+                aSymbol->AddDrawItem( dynamic_cast<SCH_ITEM*>( textItem ) );
         }
     }
 }
@@ -944,16 +931,16 @@ LIB_SYMBOL* SCH_EASYEDA_PARSER::ParseSymbol( const VECTOR2D&              aOrigi
     std::optional<wxString> valOpt;
     wxString                symbolName = wxS( "Unknown" );
 
-    if( valOpt = get_opt( aParams, wxS( "name" ) ) )
+    if( ( valOpt = get_opt( aParams, wxS( "name" ) ) ) )
         symbolName = *valOpt;
-    else if( valOpt = get_opt( aParams, wxS( "spiceSymbolName" ) ) )
+    else if( ( valOpt = get_opt( aParams, wxS( "spiceSymbolName" ) ) ) )
         symbolName = *valOpt;
 
     wxString symbolPrefix;
 
-    if( valOpt = get_opt( aParams, wxS( "pre" ) ) )
+    if( ( valOpt = get_opt( aParams, wxS( "pre" ) ) ) )
         symbolPrefix = *valOpt;
-    else if( valOpt = get_opt( aParams, wxS( "spicePre" ) ) )
+    else if( ( valOpt = get_opt( aParams, wxS( "spicePre" ) ) ) )
         symbolPrefix = *valOpt;
 
     LIB_ID libId = EasyEdaToKiCadLibID( wxEmptyString, symbolName );
@@ -971,16 +958,16 @@ LIB_SYMBOL* SCH_EASYEDA_PARSER::ParseSymbol( const VECTOR2D&              aOrigi
         if( srcName == wxS( "Datasheet" ) )
             srcName = wxS( "link" );
 
-        if( valOpt = get_opt( aParams, srcName ) )
+        if( ( valOpt = get_opt( aParams, srcName ) ) )
         {
             if( valOpt->empty() )
                 continue;
 
-            LIB_FIELD* fd = ksymbol->FindField( attrName, true );
+            SCH_FIELD* fd = ksymbol->FindField( attrName, true );
 
             if( !fd )
             {
-                fd = new LIB_FIELD( ksymbol->GetNextAvailableFieldId(), attrName );
+                fd = new SCH_FIELD( ksymbol.get(), ksymbol->GetNextAvailableFieldId(), attrName );
                 ksymbol->AddField( fd );
             }
 
@@ -1017,7 +1004,7 @@ std::pair<LIB_SYMBOL*, bool> SCH_EASYEDA_PARSER::MakePowerSymbol( const wxString
     ksymbol->SetShowPinNames( false );
     ksymbol->SetShowPinNumbers( false );
 
-    std::unique_ptr<LIB_PIN> pin = std::make_unique<LIB_PIN>( ksymbol.get() );
+    std::unique_ptr<SCH_PIN> pin = std::make_unique<SCH_PIN>( ksymbol.get() );
 
     pin->SetName( aNetname );
     pin->SetNumber( wxS( "1" ) );
@@ -1429,7 +1416,7 @@ void SCH_EASYEDA_PARSER::ParseSchematic( SCHEMATIC* aSchematic, SCH_SHEET* aRoot
 
             textItem->SetTextSize( VECTOR2I( ktextSize, ktextSize ) );
 
-            TransformTextToBaseline( textItem.get(), baselineAlign, false );
+            TransformTextToBaseline( textItem.get(), baselineAlign );
 
             createdItems.push_back( std::move( textItem ) );
         }
@@ -1505,16 +1492,16 @@ void SCH_EASYEDA_PARSER::ParseSchematic( SCHEMATIC* aSchematic, SCH_SHEET* aRoot
                             {
                                 // Lines need special handling for some reason
                                 aSchItem->SetFlags( STARTPOINT );
-                                aSchItem->Rotate( RelPos( cmdAround ) );
+                                aSchItem->Rotate( RelPos( cmdAround ), false );
                                 aSchItem->ClearFlags( STARTPOINT );
 
                                 aSchItem->SetFlags( ENDPOINT );
-                                aSchItem->Rotate( RelPos( cmdAround ) );
+                                aSchItem->Rotate( RelPos( cmdAround ), false );
                                 aSchItem->ClearFlags( ENDPOINT );
                             }
                             else
                             {
-                                aSchItem->Rotate( RelPos( cmdAround ) );
+                                aSchItem->Rotate( RelPos( cmdAround ), false );
                             }
                         }
                     }
@@ -1607,16 +1594,17 @@ void SCH_EASYEDA_PARSER::ParseSchematic( SCHEMATIC* aSchematic, SCH_SHEET* aRoot
                     else
                     {
                         std::unique_ptr<SCH_BITMAP> bitmap = std::make_unique<SCH_BITMAP>();
+                        REFERENCE_IMAGE&            refImage = bitmap->GetReferenceImage();
 
                         wxImage::SetDefaultLoadFlags( wxImage::GetDefaultLoadFlags()
                                                       & ~wxImage::Load_Verbose );
 
-                        if( bitmap->ReadImageFile( buf ) )
+                        if( refImage.ReadImageFile( buf ) )
                         {
                             VECTOR2D kcenter = kstart + ksize / 2;
 
-                            double scaleFactor = ScaleSize( size.x ) / bitmap->GetSize().x;
-                            bitmap->SetImageScale( scaleFactor );
+                            double scaleFactor = ScaleSize( size.x ) / refImage.GetSize().x;
+                            refImage.SetImageScale( scaleFactor );
                             bitmap->SetPosition( kcenter );
 
                             applyTransform( bitmap.get() );
