@@ -2,7 +2,7 @@
  * This program source code file is part of KICAD, a free EDA CAD application.
  *
  * Copyright (C) 2020 CERN
- * Copyright (C) 2021-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  *
  * This program is free software; you can redistribute it and/or
@@ -30,11 +30,13 @@
 #include <properties/property_mgr.h>
 #include <sch_commit.h>
 #include <sch_edit_frame.h>
+#include <symbol_edit_frame.h>
+#include <symbol_viewer_frame.h>
 #include <schematic.h>
 #include <settings/color_settings.h>
 #include <string_utils.h>
 #include <tool/tool_manager.h>
-#include <tools/ee_selection_tool.h>
+#include <tools/sch_selection_tool.h>
 
 
 SCH_PROPERTIES_PANEL::SCH_PROPERTIES_PANEL( wxWindow* aParent, SCH_BASE_FRAME* aFrame ) :
@@ -101,8 +103,8 @@ SCH_PROPERTIES_PANEL::~SCH_PROPERTIES_PANEL()
 
 void SCH_PROPERTIES_PANEL::UpdateData()
 {
-    EE_SELECTION_TOOL* selectionTool = m_frame->GetToolManager()->GetTool<EE_SELECTION_TOOL>();
-    const SELECTION& selection = selectionTool->GetSelection();
+    SCH_SELECTION_TOOL* selectionTool = m_frame->GetToolManager()->GetTool<SCH_SELECTION_TOOL>();
+    const SELECTION&    selection = selectionTool->GetSelection();
 
     // Will actually just be updatePropertyValues() if selection hasn't changed
     rebuildProperties( selection );
@@ -111,15 +113,10 @@ void SCH_PROPERTIES_PANEL::UpdateData()
 
 void SCH_PROPERTIES_PANEL::AfterCommit()
 {
-    EE_SELECTION_TOOL* selectionTool = m_frame->GetToolManager()->GetTool<EE_SELECTION_TOOL>();
-    const SELECTION& selection = selectionTool->GetSelection();
+    SCH_SELECTION_TOOL* selectionTool = m_frame->GetToolManager()->GetTool<SCH_SELECTION_TOOL>();
+    const SELECTION&    selection = selectionTool->GetSelection();
 
     rebuildProperties( selection );
-
-    CallAfter( [this]()
-               {
-                   m_frame->GetCanvas()->SetFocus();
-               } );
 }
 
 
@@ -139,9 +136,9 @@ wxPGProperty* SCH_PROPERTIES_PANEL::createPGProperty( const PROPERTY_BASE* aProp
 
 PROPERTY_BASE* SCH_PROPERTIES_PANEL::getPropertyFromEvent( const wxPropertyGridEvent& aEvent ) const
 {
-    EE_SELECTION_TOOL* selectionTool = m_frame->GetToolManager()->GetTool<EE_SELECTION_TOOL>();
-    const SELECTION& selection = selectionTool->GetSelection();
-    SCH_ITEM* firstItem = static_cast<SCH_ITEM*>( selection.Front() );
+    SCH_SELECTION_TOOL* selectionTool = m_frame->GetToolManager()->GetTool<SCH_SELECTION_TOOL>();
+    const SELECTION&    selection = selectionTool->GetSelection();
+    SCH_ITEM*           firstItem = static_cast<SCH_ITEM*>( selection.Front() );
 
     wxCHECK_MSG( firstItem, nullptr,
                  wxT( "getPropertyFromEvent for a property with nothing selected!") );
@@ -157,9 +154,9 @@ PROPERTY_BASE* SCH_PROPERTIES_PANEL::getPropertyFromEvent( const wxPropertyGridE
 
 void SCH_PROPERTIES_PANEL::valueChanging( wxPropertyGridEvent& aEvent )
 {
-    EE_SELECTION_TOOL* selectionTool = m_frame->GetToolManager()->GetTool<EE_SELECTION_TOOL>();
-    const SELECTION& selection = selectionTool->GetSelection();
-    EDA_ITEM* item = selection.Front();
+    SCH_SELECTION_TOOL* selectionTool = m_frame->GetToolManager()->GetTool<SCH_SELECTION_TOOL>();
+    const SELECTION&    selection = selectionTool->GetSelection();
+    EDA_ITEM*           item = selection.Front();
 
     PROPERTY_BASE* property = getPropertyFromEvent( aEvent );
     wxCHECK( property, /* void */ );
@@ -175,19 +172,20 @@ void SCH_PROPERTIES_PANEL::valueChanging( wxPropertyGridEvent& aEvent )
         aEvent.Veto();
         return;
     }
+
+    aEvent.Skip();
 }
 
 
 void SCH_PROPERTIES_PANEL::valueChanged( wxPropertyGridEvent& aEvent )
 {
-    EE_SELECTION_TOOL* selectionTool = m_frame->GetToolManager()->GetTool<EE_SELECTION_TOOL>();
-    const SELECTION& selection = selectionTool->GetSelection();
-
-    PROPERTY_BASE* property = getPropertyFromEvent( aEvent );
+    SCH_SELECTION_TOOL* selectionTool = m_frame->GetToolManager()->GetTool<SCH_SELECTION_TOOL>();
+    const SELECTION&    selection = selectionTool->GetSelection();
+    PROPERTY_BASE*      property = getPropertyFromEvent( aEvent );
     wxCHECK( property, /* void */ );
 
-    wxVariant newValue = aEvent.GetPropertyValue();
-    SCH_COMMIT changes( m_frame );
+    wxVariant   newValue = aEvent.GetPropertyValue();
+    SCH_COMMIT  changes( m_frame );
     SCH_SCREEN* screen = m_frame->GetScreen();
 
     PROPERTY_COMMIT_HANDLER handler( &changes );
@@ -197,29 +195,58 @@ void SCH_PROPERTIES_PANEL::valueChanged( wxPropertyGridEvent& aEvent )
         SCH_ITEM* item = static_cast<SCH_ITEM*>( edaItem );
         changes.Modify( item, screen );
         item->Set( property, newValue );
+
+        if( SCH_SYMBOL* symbol = dynamic_cast<SCH_SYMBOL*>( item ) )
+            symbol->SyncOtherUnits( symbol->Schematic()->CurrentSheet(), changes, property );
     }
 
-    changes.Push( _( "Change property" ) );
+    changes.Push( _( "Edit Properties" ) );
     m_frame->Refresh();
 
     // Perform grid updates as necessary based on value change
     AfterCommit();
+
+    aEvent.Skip();
 }
 
 
-void SCH_PROPERTIES_PANEL::LanguageChanged()
+void SCH_PROPERTIES_PANEL::OnLanguageChanged( wxCommandEvent& aEvent )
 {
+    PROPERTIES_PANEL::OnLanguageChanged( aEvent );
     updateFontList();
+
+    aEvent.Skip();
 }
 
 
 void SCH_PROPERTIES_PANEL::updateFontList()
 {
     wxPGChoices fonts;
+    const std::vector<wxString>* fontFiles = nullptr;
+
+    if( m_frame->GetFrameType() == FRAME_SCH && m_frame->GetScreen() && m_frame->GetScreen()->Schematic() )
+    {
+        fontFiles = m_frame->GetScreen()->Schematic()->GetEmbeddedFiles()->GetFontFiles();
+    }
+    else if( m_frame->GetFrameType() == FRAME_SCH_SYMBOL_EDITOR )
+    {
+        SYMBOL_EDIT_FRAME* symbolFrame = static_cast<SYMBOL_EDIT_FRAME*>( m_frame );
+
+        if( symbolFrame->GetCurSymbol() )
+            fontFiles = symbolFrame->GetCurSymbol()->GetEmbeddedFiles()->UpdateFontFiles();
+    }
+    else if( m_frame->GetFrameType() == FRAME_SCH_VIEWER )
+    {
+        SYMBOL_VIEWER_FRAME* symbolFrame = static_cast<SYMBOL_VIEWER_FRAME*>( m_frame );
+
+        if( symbolFrame->GetSelectedSymbol() )
+            fontFiles = symbolFrame->GetSelectedSymbol()->GetEmbeddedFiles()->UpdateFontFiles();
+    }
 
     // Regnerate font names
     std::vector<std::string> fontNames;
-    Fontconfig()->ListFonts( fontNames, std::string( Pgm().GetLanguageTag().utf8_str() ) );
+    Fontconfig()->ListFonts( fontNames, std::string( Pgm().GetLanguageTag().utf8_str() ),
+                             fontFiles );
 
     fonts.Add( _( "Default Font" ), -1 );
     fonts.Add( KICAD_FONT_NAME, -2 );
