@@ -775,6 +775,13 @@ void KICAD_MANAGER_FRAME::OpenJobsFile( const wxFileName& aFileName, bool aCreat
 
         jobsFile->LoadFromFile();
 
+        if( aCreate && !aFileName.FileExists() )
+        {
+            JOBSET_DESTINATION* dest = jobsFile->AddNewDestination( JOBSET_DESTINATION_T::FOLDER );
+            dest->m_outputHandler->SetOutputPath( aFileName.GetName() );
+            jobsFile->SaveToFile( wxEmptyString, true );
+        }
+
         PANEL_JOBSET* jobPanel = new PANEL_JOBSET( m_notebook, this, std::move( jobsFile ) );
         jobPanel->SetProjectTied( true );
         jobPanel->SetClosable( true );
@@ -1016,6 +1023,8 @@ void KICAD_MANAGER_FRAME::CommonSettingsChanged( int aFlags )
         onToolbarSizeChanged();
         m_lastToolbarIconSize = settings->m_Appearance.toolbar_icon_size;
     }
+
+    m_leftWin->ReCreateTreePrj();
 }
 
 
