@@ -364,6 +364,7 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( BOARD_COMMIT& aCommit, B
 {
     aCommit.Modify( aItem );
 
+    PCB_FIELD*          field = dynamic_cast<PCB_FIELD*>( aItem );
     PCB_TEXT*           text = dynamic_cast<PCB_TEXT*>( aItem );
     PCB_SHAPE*          shape = dynamic_cast<PCB_SHAPE*>( aItem );
     PCB_DIMENSION_BASE* dimension = dynamic_cast<PCB_DIMENSION_BASE*>( aItem );
@@ -412,9 +413,6 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( BOARD_COMMIT& aCommit, B
 
             if( parentFP )
             {
-                if( m_visible->Get3StateValue() != wxCHK_UNDETERMINED )
-                    text->SetVisible( m_visible->GetValue() );
-
                 if( m_keepUpright->Get3StateValue() != wxCHK_UNDETERMINED )
                     text->SetKeepUpright( m_keepUpright->GetValue() );
 
@@ -423,6 +421,12 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( BOARD_COMMIT& aCommit, B
             }
         }
 
+        if( field )
+        {
+            if( m_visible->Get3StateValue() != wxCHK_UNDETERMINED )
+                field->SetVisible( m_visible->GetValue() );
+        }
+
         if( !m_lineWidth.IsIndeterminate() )
         {
             if( shape )
