@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,7 +25,7 @@
 #include <pgm_base.h>
 #include <kiface_base.h>
 #include <confirm.h>
-#include <string_utils.h>
+#include <kidialog.h>
 #include <macros.h>
 #include <pcb_edit_frame.h>
 #include <eda_list_dialog.h>
@@ -38,11 +38,10 @@
 #include <tools/board_editor_control.h>
 #include <tools/pad_tool.h>
 #include <footprint.h>
+#include <zone.h>
 #include <pcb_group.h>
-#include <board_commit.h>
 #include <footprint_edit_frame.h>
 #include <wildcards_and_files_ext.h>
-#include <pcb_io/kicad_legacy/pcb_io_kicad_legacy.h>
 #include <pcb_io/kicad_sexpr/pcb_io_kicad_sexpr.h>
 #include <env_paths.h>
 #include <paths.h>
@@ -51,6 +50,7 @@
 #include <project/project_file.h>
 #include <footprint_editor_settings.h>
 #include <footprint_viewer_frame.h>
+#include <view/view_controls.h>
 #include <wx/choicdlg.h>
 #include <wx/filedlg.h>
 #include <wx/fswatcher.h>
@@ -527,10 +527,19 @@ bool FOOTPRINT_EDIT_FRAME::DeleteFootprintFromLibrary( const LIB_ID& aFPID, bool
 
     wxString nickname = aFPID.GetLibNickname();
     wxString fpname = aFPID.GetLibItemName();
+    wxString libfullname;
 
     // Legacy libraries are readable, but modifying legacy format is not allowed
     // So prompt the user if he try to delete a footprint from a legacy lib
-    wxString libfullname = PROJECT_PCB::PcbFootprintLibs( &Prj() )->FindRow( nickname )->GetFullURI();
+    try
+    {
+        libfullname = PROJECT_PCB::PcbFootprintLibs( &Prj() )->FindRow( nickname )->GetFullURI();
+    }
+    catch( ... )
+    {
+        // If we can't find the nickname, stop here
+        return false;
+    }
 
     if( PCB_IO_MGR::GuessPluginTypeFromLibPath( libfullname ) == PCB_IO_MGR::LEGACY )
     {
@@ -597,6 +606,13 @@ void PCB_EDIT_FRAME::ExportFootprintsToLibrary( bool aStoreInNewLib, const wxStr
                     parentGroup->RemoveItem( aFootprint );
             };
 
+    auto resetZones =
+            []( FOOTPRINT* aFootprint )
+            {
+                for( ZONE* zone : aFootprint->Zones() )
+                    zone->Move( -aFootprint->GetPosition() );
+            };
+
     if( !aStoreInNewLib )
     {
         // The footprints are saved in an existing .pretty library in the fp lib table
@@ -625,6 +641,7 @@ void PCB_EDIT_FRAME::ExportFootprintsToLibrary( bool aStoreInNewLib, const wxStr
                     // Reset reference designator and group membership before saving
                     resetReference( fpCopy );
                     resetGroup( fpCopy );
+                    resetZones( fpCopy );
 
                     tbl->FootprintSave( nickname, fpCopy, true );
 
@@ -669,6 +686,7 @@ void PCB_EDIT_FRAME::ExportFootprintsToLibrary( bool aStoreInNewLib, const wxStr
 
         PCB_IO_MGR::PCB_FILE_T piType = PCB_IO_MGR::KICAD_SEXP;
         IO_RELEASER<PCB_IO>    pi( PCB_IO_MGR::PluginFind( piType ) );
+        std::map<std::string, UTF8> options { { "skip_cache_validation", "1" } }; // Skip cache validation -- we just created it
 
         for( FOOTPRINT* footprint : GetBoard()->Footprints() )
         {
@@ -681,8 +699,9 @@ void PCB_EDIT_FRAME::ExportFootprintsToLibrary( bool aStoreInNewLib, const wxStr
                     // Reset reference designator and group membership before saving
                     resetReference( fpCopy );
                     resetGroup( fpCopy );
+                    resetZones( fpCopy );
 
-                    pi->FootprintSave( libPath, fpCopy );
+                    pi->FootprintSave( libPath, fpCopy, &options );
 
                     delete fpCopy;
                 }
@@ -915,6 +934,28 @@ bool FOOTPRINT_EDIT_FRAME::SaveFootprintToBoard( bool aAddNew )
                 fixUuid( const_cast<KIID&>( aChild->m_Uuid ) );
             } );
 
+    // Right now, we only show the "Unconnected" net in the footprint editor, but this is still
+    // referenced in the footprint.  So we need to update the net pointers in the footprint to
+    // point to the nets in the main board.
+    newFootprint->RunOnDescendants(
+            [&]( BOARD_ITEM* aChild )
+            {
+                if( BOARD_CONNECTED_ITEM* conn = dynamic_cast<BOARD_CONNECTED_ITEM*>( aChild ) )
+                {
+                    NETINFO_ITEM* net = conn->GetNet();
+                    auto& netmap = mainpcb->GetNetInfo().NetsByName();
+
+                    if( net )
+                    {
+                        auto it = netmap.find( net->GetNetname() );
+
+                        if( it != netmap.end() )
+                            conn->SetNet( it->second );
+                    }
+
+                }
+            } );
+
     BOARD_DESIGN_SETTINGS& bds = m_pcb->GetDesignSettings();
 
     newFootprint->ApplyDefaultSettings( *m_pcb, bds.m_StyleFPFields, bds.m_StyleFPText,
@@ -1195,97 +1236,45 @@ bool FOOTPRINT_EDIT_FRAME::RevertFootprint()
 }
 
 
-class NEW_FP_DIALOG : public WX_TEXT_ENTRY_DIALOG
+FOOTPRINT* PCB_BASE_FRAME::CreateNewFootprint( wxString aFootprintName, const wxString& aLibName )
 {
-public:
-    NEW_FP_DIALOG( PCB_BASE_FRAME* aParent, const wxString& aName, int aFootprintType,
-                   std::function<bool( wxString newName )> aValidator ) :
-            WX_TEXT_ENTRY_DIALOG( aParent, _( "Enter footprint name:" ), _( "New Footprint" ),
-                                  aName, _( "Footprint type:" ),
-                                  { _( "Through hole" ), _( "SMD" ), _( "Other" ) },
-                                  aFootprintType ),
-            m_validator( std::move( aValidator ) )
-    { }
+    if( aFootprintName.IsEmpty() )
+        aFootprintName = _( "Untitled" );
 
-    wxString GetFPName()
+    int footprintAttrs = FP_SMD;
+
+    if( !aLibName.IsEmpty() )
     {
-        wxString name = m_textCtrl->GetValue();
-        name.Trim( true ).Trim( false );
-        return name;
-    }
+        FP_LIB_TABLE* tbl = PROJECT_PCB::PcbFootprintLibs( &Prj() );
+        wxArrayString fpnames;
+        wxString      baseName = aFootprintName;
+        int           idx = 1;
 
-protected:
-    bool TransferDataFromWindow() override
-    {
-        return m_validator( GetFPName() );
-    }
+        // Make sure the name is unique
+        while( tbl->FootprintExists( aLibName, aFootprintName ) )
+            aFootprintName = baseName + wxString::Format( wxS( "_%d" ), idx++ );
 
-private:
-    std::function<bool( wxString newName )> m_validator;
-};
-
-
-FOOTPRINT* PCB_BASE_FRAME::CreateNewFootprint( const wxString& aFootprintName,
-                                               const wxString& aLibName, bool aQuiet )
-{
-    FP_LIB_TABLE* tbl = PROJECT_PCB::PcbFootprintLibs( &Prj() );
-    wxString      footprintName = aFootprintName;
-    wxString      msg;
-
-    // Static to store user preference for a session
-    static int footprintType = 1;
-    int footprintTranslated = FP_SMD;
-
-    // Ask for the new footprint name
-    if( footprintName.IsEmpty() && !aQuiet )
-    {
-        NEW_FP_DIALOG dlg( this, footprintName, footprintType,
-                [&]( wxString newName )
-                {
-                    if( newName.IsEmpty() )
-                    {
-                        wxMessageBox( _( "Footprint must have a name." ) );
-                        return false;
-                    }
-
-                    if( !aLibName.IsEmpty() && tbl->FootprintExists( aLibName, newName ) )
-                    {
-                        msg = wxString::Format( _( "Footprint '%s' already exists in library '%s'." ),
-                                                newName, aLibName );
-
-                        KIDIALOG errorDlg( this, msg, _( "Confirmation" ),
-                                           wxOK | wxCANCEL | wxICON_WARNING );
-                        errorDlg.SetOKLabel( _( "Overwrite" ) );
-
-                        return errorDlg.ShowModal() == wxID_OK;
-                    }
-
-                    return true;
-                } );
-
-        dlg.SetTextValidator( FOOTPRINT_NAME_VALIDATOR( &footprintName ) );
-
-        if( dlg.ShowModal() != wxID_OK )
-            return nullptr;    //Aborted by user
-
-        footprintName = dlg.GetFPName();
-        footprintType = dlg.GetChoice();
-
-        switch( footprintType )
+        // Try to infer the footprint attributes from an existing footprint in the library
+        try
         {
-        case 0:  footprintTranslated = FP_THROUGH_HOLE; break;
-        case 1:  footprintTranslated = FP_SMD;          break;
-        default: footprintTranslated = 0;               break;
+            tbl->FootprintEnumerate( fpnames, aLibName, true );
+
+            if( !fpnames.empty() )
+                footprintAttrs = tbl->FootprintLoad( aLibName, fpnames.Last() )->GetAttributes();
+        }
+        catch( ... )
+        {
+            // best efforts
         }
     }
 
-    // Creates the new footprint and add it to the head of the linked list of footprints
+    // Create the new footprint and add it to the head of the linked list of footprints
     FOOTPRINT* footprint = new FOOTPRINT( GetBoard() );
 
     // Update its name in lib
-    footprint->SetFPID( LIB_ID( wxEmptyString, footprintName ) );
+    footprint->SetFPID( LIB_ID( wxEmptyString, aFootprintName ) );
 
-    footprint->SetAttributes( footprintTranslated );
+    footprint->SetAttributes( footprintAttrs );
 
     PCB_LAYER_ID txt_layer;
     VECTOR2I     default_pos;
@@ -1293,7 +1282,7 @@ FOOTPRINT* PCB_BASE_FRAME::CreateNewFootprint( const wxString& aFootprintName,
 
     footprint->Reference().SetText( settings.m_DefaultFPTextItems[0].m_Text );
     footprint->Reference().SetVisible( settings.m_DefaultFPTextItems[0].m_Visible );
-    txt_layer = (PCB_LAYER_ID) settings.m_DefaultFPTextItems[0].m_Layer;
+    txt_layer = settings.m_DefaultFPTextItems[0].m_Layer;
     footprint->Reference().SetLayer( txt_layer );
     default_pos.y -= settings.GetTextSize( txt_layer ).y / 2;
     footprint->Reference().SetPosition( default_pos );
@@ -1301,7 +1290,7 @@ FOOTPRINT* PCB_BASE_FRAME::CreateNewFootprint( const wxString& aFootprintName,
 
     footprint->Value().SetText( settings.m_DefaultFPTextItems[1].m_Text );
     footprint->Value().SetVisible( settings.m_DefaultFPTextItems[1].m_Visible );
-    txt_layer = (PCB_LAYER_ID) settings.m_DefaultFPTextItems[1].m_Layer;
+    txt_layer = settings.m_DefaultFPTextItems[1].m_Layer;
     footprint->Value().SetLayer( txt_layer );
     default_pos.y += settings.GetTextSize( txt_layer ).y / 2;
     footprint->Value().SetPosition( default_pos );
@@ -1311,7 +1300,6 @@ FOOTPRINT* PCB_BASE_FRAME::CreateNewFootprint( const wxString& aFootprintName,
     {
         PCB_TEXT* textItem = new PCB_TEXT( footprint );
         textItem->SetText( settings.m_DefaultFPTextItems[i].m_Text );
-        textItem->SetVisible( settings.m_DefaultFPTextItems[i].m_Visible );
         txt_layer = (PCB_LAYER_ID) settings.m_DefaultFPTextItems[i].m_Layer;
         textItem->SetLayer( txt_layer );
         default_pos.y += settings.GetTextSize( txt_layer ).y / 2;
@@ -1321,10 +1309,10 @@ FOOTPRINT* PCB_BASE_FRAME::CreateNewFootprint( const wxString& aFootprintName,
     }
 
     if( footprint->GetReference().IsEmpty() )
-        footprint->SetReference( footprintName );
+        footprint->SetReference( aFootprintName );
 
     if( footprint->GetValue().IsEmpty() )
-        footprint->SetValue( footprintName );
+        footprint->SetValue( aFootprintName );
 
     footprint->RunOnDescendants(
             [&]( BOARD_ITEM* aChild )
