@@ -791,6 +791,11 @@ void PCB_IO_EASYEDA_PARSER::ParseToBoardItemContainer(
             {
                 text = footprint->GetField( VALUE_FIELD );
             }
+            else if( arr[12] == wxS( "none" ) )
+            {
+                text = new PCB_FIELD( aContainer, -1 );
+                static_cast<PCB_FIELD*>( text )->SetVisible( false );
+            }
             else
             {
                 text = new PCB_TEXT( aContainer );
@@ -826,8 +831,6 @@ void PCB_IO_EASYEDA_PARSER::ParseToBoardItemContainer(
 
             //arr[11] // Geometry data
 
-            text->SetVisible( arr[12] != wxS( "none" ) );
-
             wxString font = arr[14];
             if( !font.IsEmpty() )
                 text->SetFont( KIFONT::FONT::GetFont( font ) );
@@ -973,7 +976,7 @@ void PCB_IO_EASYEDA_PARSER::ParseToBoardItemContainer(
             else if( klayer == B_Cu )
             {
                 pad->SetLayer( B_Cu );
-                pad->SetLayerSet( PAD::SMDMask().Flip() );
+                pad->SetLayerSet( PAD::SMDMask().FlipStandardLayers() );
                 pad->SetAttribute( PAD_ATTRIB::SMD );
             }
             else if( elayer == wxS( "11" ) )
